package es.prodevelop.tilecache.renderer.wms;

import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.gvsig.remoteclient.utils.Utilities;
import org.gvsig.remoteclient.wms.WMSStatus;

import es.prodevelop.gvsig.mini.common.CompatManager;
import es.prodevelop.gvsig.mini.exceptions.BaseException;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Point;
import es.prodevelop.gvsig.mini.projection.TileConversor;
import es.prodevelop.gvsig.mini.wms.WMSCancellable;
import es.prodevelop.tilecache.renderer.MapRenderer;
import es.prodevelop.tilecache.renderer.TMSRenderer;
import es.prodevelop.tilecache.util.Tags;

public class OSRenderer extends TMSRenderer {

	private final static Logger logger = Logger.getLogger(TMSRenderer.class
			.getName());

	String version = "";
	String[] keys = null;
	String[] URLs = null;

	String firstPart = "";
	String secondPart = "";

	final String[] baseKeys = new String[] {
			"88013ECD7A8F379FE0405F0ACA607F60",
			"8824326E25F94E17E0405F0AC86047BC" };
	final String[] baesURLs = new String[] { "http://www.prodevelop.es",
			"http://www.gvsigmini.org" };

	int index = 0;

	WMSCancellable neverCanceled = new WMSCancellable();
	public static final int DEFAULT_MAX_ZOOM_LEVEL = 20;

	OSRenderer(String[] baseUrl, String name, String imageFilenameEnding,
			int zoomMax, int zoomMin, int tileSizePX, Extent bbox, String srs,
			String version, double[] resolutions, String keys, String urls, Extent offExtent) throws Exception  {
		super(baseUrl, name, imageFilenameEnding, zoomMax, zoomMin, tileSizePX,
				MapRenderer.OS_RENDERER, bbox.getLefBottomCoordinate(), bbox,
				srs, resolutions, offExtent);
		// try {
		//CompatManager.getInstance().getRegisteredLogHandler().configureLogger(
		// logger);
		// } catch (BaseException e1) {
		// // TODO Auto-generated catch block
		// e1.printStackTrace();
		// }
		try {
			this.version = version;

			this.setKeys(keys.split(":"));
			this.setKeyURLs(urls.split("\\|"));
		} catch (Exception e) {
			logger.log(Level.SEVERE, "OSRenderer: ", e);
			if (e instanceof BaseException) {
				throw (BaseException)e;
			}
		}
	}

	/**
	 * Instantiates a OSRenderer. @see Layers and layers.txt
	 * 
	 * @param props
	 *            The properties of the OSRenderer
	 * @param layerName
	 *            The name of the layer
	 * @return A OSRenderer
	 */
	public static OSRenderer getOSRenderer(String[] props, String layerName) {
		try {

			TMSRenderer w = (TMSRenderer) TMSRenderer.getTMSRenderer(props,
					layerName);

			OSRenderer wmsR = new OSRenderer(new String[] { w.getURL() }, w
					.getNAME(), w.getIMAGE_FILENAMEENDING(), w
					.getZOOM_MAXLEVEL(), w.getZoomMinLevel(), w
					.getMAPTILE_SIZEPX(), w.getExtent(), w.getSRS(), props[15],
					w.resolutions, props[16], props[17], null);

			return wmsR;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getOSRenderer: ", e);
			return null;
		}
	}

	public void setZoomLevel(int zoomLevel) {
		super.setZoomLevel(zoomLevel);
		if (zoomLevel > 8) {
			this.setMAPTILE_SIZEPX(250);
		} else {
			this.setMAPTILE_SIZEPX(200);
		}
		Tags.DEFAULT_TILE_SIZE = getMAPTILE_SIZEPX();
		es.prodevelop.gvsig.mini.utiles.Tags.DEFAULT_TILE_SIZE = getMAPTILE_SIZEPX();
		TileConversor.pixelsPerTile = getMAPTILE_SIZEPX();

	}

	public static OSRenderer getOSRenderer(String[] baseUrl, String name,
			String imageFilenameEnding, int zoomMax, int zoomMin,
			int tileSizePX, Extent bbox, String srs, String version,
			double[] resolutions, String keys, String urls, Extent offExtent) {
		try {
			OSRenderer r = new OSRenderer(baseUrl, name, imageFilenameEnding,
					zoomMax, zoomMin, tileSizePX, bbox, srs, version,
					resolutions, keys, urls, offExtent);
			return r;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getOSRenderer: ", e);
			return null;
		}
	}

	private void calculateResolutions(double size, int zoomMax) {
		try {
			resolutions = new double[zoomMax + 1];
			double iniRes = size / this.getMAPTILE_SIZEPX();
			for (int i = 0; i <= zoomMax; i++) {
				resolutions[i] = iniRes;
				size /= 2;
				iniRes = size / this.getMAPTILE_SIZEPX();
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "calculateResolutions: ", e);
		}
	}

	public String getBASEURL() {
		String baseURL = super.getBASEURL();

		if (index >= keys.length) {
			index = 0;
		}

		String key = keys[index];
		String keyURL = URLs[index];

		if (firstPart.compareTo("") == 0 || secondPart.compareTo("") == 0) {
			firstPart = baseURL.substring(0, baseURL.indexOf("#"));
			secondPart = baseURL.substring(baseURL.indexOf("#") + 1, baseURL
					.length());

			secondPart = secondPart.substring(secondPart.indexOf("#") + 1,
					secondPart.length());

			secondPart = secondPart.substring(0, secondPart.indexOf("#"));

		}

		return new StringBuffer().append(firstPart).append(key).append(
				secondPart).append(keyURL).append("/maps/map.htm").toString();
	}

	@Override
	public String getTileURLString(int[] tileID, int zoomLevel) {
		try {

			// logger.debug("tile: " + tileID[0] + ", " + tileID[1]);
			final WMSStatus status = new WMSStatus();

			status.addLayerName(String.valueOf((int) resolutions[zoomLevel]));

			status.setSrs(this.SRS);
			status.setFormat(this.getIMAGE_FILENAMEENDING());

			// if (CRSFactory.getCRS(this.SRS).getUnitsAbbrev().compareTo("g") 
			// == 0) {
			// BBOX = TileConversor.tileBoundsG(tileID[1], tileID[0],
			// resolutions[zoomLevel]);
			// } else {

			Extent BBOX = TileConversor.tileMeterBounds(tileID[1], tileID[0],
					resolutions[zoomLevel]);
			// }

			status.setExtent(BBOX);
			status.setWidth(this.getMAPTILE_SIZEPX());
			status.setHeight(this.getMAPTILE_SIZEPX());

			final StringBuffer req = new StringBuffer();
			status.setDisconnectedServerURL(this.getBASEURL());
			// String info = FMapWMSDriverFactory.getFMapDriverForURL(
			// new URL(this.getBASEURL())).getFeatureInfo(status, 0, 0,
			// 10, this.neverCanceled);
			req.append(this.getBASEURL());
			String symbol = "?";
			if (this.getBASEURL().charAt(this.getBASEURL().length() - 1) == '?') {
				symbol = "";
			}

			req.append(symbol).append("REQUEST=GetMap&SERVICE=WMS&VERSION=")
					.append(version).append("&");
			req.append(getPartialQuery(status));
			System.out.println(req.toString());
			index++;
			return req.toString().replaceAll(" ", "%20");

		} catch (Exception e) {
			logger.log(Level.SEVERE, "getTileURLString: ", e);
			return null;
		}
	}

	protected String getPartialQuery(WMSStatus status) {
		final String fmt = status.getFormat();
		final StringBuffer req = new StringBuffer();
		req.append("LAYERS=" + Utilities.Vector2CS(status.getLayerNames()))
				.append("&SRS=" + status.getSrs()).append(
						"&BBOX=" + status.getExtent().getMinX() + ",").append(
						status.getExtent().getMinY() + ",").append(
						status.getExtent().getMaxX() + ",").append(
						status.getExtent().getMaxY()).append(
						"&WIDTH=" + status.getWidth()).append(
						"&HEIGHT=" + status.getHeight()).append(
						"&FORMAT=" + fmt).append("&STYLES=");

		Vector v = status.getStyles();
		if (v != null && v.size() > 0)
			req.append(Utilities.Vector2CS(v));
		v = status.getDimensions();
		if (v != null && v.size() > 0)
			req.append("&" + Utilities.Vector2URLParamString(v));
		if (status.getTransparency()) {
			req.append("&TRANSPARENT=TRUE");
		}
		return req.toString();
	}

	@Override
	public int isTMS() {
		return -1;
	}

	@Override
	public String toString() {
		StringBuffer layer = new StringBuffer();
		try {
			String base = super.toString();

			StringBuffer layerNames = new StringBuffer().append(" ");

			layer.append(base).append(",").append(layerNames);
			layer.append(",").append(version);

			int size = keys.length;
			for (int i = 0; i < size; i++) {
				if (i == 0)
					layer.append(",");
				layer.append(keys[i]);
				if (i < size - 1) {
					layer.append(":");
				}
			}

			size = URLs.length;
			for (int i = 0; i < size; i++) {
				if (i == 0)
					layer.append(",");
				layer.append(URLs[i]);
				if (i < size - 1) {
					layer.append("|");
				}
			}
			return layer.toString();

		} catch (Exception e) {
			logger.log(Level.SEVERE, "toString: ", e);
			return null;
		}
	}

	/**
	 * Instantiates a WMSStatus filling the properties: layerNames, SRS, format,
	 * width, height, disconnectedServerURL. The bounding box has to be filled
	 * separatelly
	 * 
	 * @return The WMSStatus with the current parameters of the OSRenderer
	 */
	public WMSStatus buildWMSStatus() {
		WMSStatus status = null;
		try {
			status = new WMSStatus();

			status.addLayerName(String
					.valueOf((int) resolutions[getZoomLevel()]));

			status.setSrs(this.SRS);
			status.setFormat(this.getIMAGE_FILENAMEENDING());

			status.setWidth(this.getMAPTILE_SIZEPX());
			status.setHeight(this.getMAPTILE_SIZEPX());

			status.setDisconnectedServerURL(this.getBASEURL());
		} catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
		}
		return status;
	}

	public void setKey(String key) throws BaseException {
		if (key != null && key.trim().compareTo("") != 0) {
			this.keys = new String[] { key };
		} else {
			throw new BaseException("Wrong key");
		}
	}

	public void setKeyURL(String keyURL) throws BaseException {
		if (keyURL != null && keyURL.trim().compareTo("") != 0) {
			this.URLs = new String[] { keyURL };
		} else {
			throw new BaseException("Wrong keyURL");
		}
	}

	public void setKeys(String[] keys) throws BaseException {
		if (keys != null && keys.length > 0) {
			this.keys = keys;
		} else {
			throw new BaseException("Wrong keys");
		}
	}

	public void setKeyURLs(String[] keyURLs) throws BaseException {
		if (keyURLs != null && keyURLs.length > 0) {
			this.URLs = keyURLs;
		} else {
			throw new BaseException("Wrong URLs");
		}
	}
	
	public void setDefaultKeysAndURLs() throws BaseException {
		this.setKeys(this.baseKeys);
		this.setKeyURLs(this.baesURLs);
	}

}
